﻿
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.Interface;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;


namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUniv.Hydrologyツール系モデルファクトリクラス</para>
	/// </summary>
	public class ToolFactory : McBasicModelFactoryBase
	{
		/// <summary><para>method outline:</para>
		/// <para>デフォルトコンストラクタ</para>
		/// </summary>
		/// <param name="">無し</param>
		/// <returns>インスタンス</returns>
		/// <exception cref="">無し</exception>
		public ToolFactory()
		{
		}

		/// <summary><para>method outline:</para>
		/// <para>ファクトリ識別子を生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> HySID csFactID = CreateFactoryID( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>ファクトリ識別子</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override HySID CreateFactoryID()
		{
			return ToolDefine.TOOL_LIB;
		}

		/// <summary><para>method outline:</para>
		/// <para>演算モデルクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McCalModel csCalModel = CreateCalModel(csModelKind) </para>
		/// </example>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>演算モデルクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCalModel CreateCalModel(HySObjectKind csModelKind)
		{
			McCalModel csCalModel = null;

			if (csModelKind == ToolDefine.FILE_IN_KIND)
			{
				csCalModel = new FileInputBasic();
			}

			else if (csModelKind == ToolDefine.FILE_IN_MATRIX_KIND)
			{
				csCalModel = new FileInputMatrix();
			}

			else if (csModelKind == ToolDefine.FILE_OUT_KIND)
			{
				csCalModel = new FileOutputBasic();
			}
			
			else if (csModelKind == ToolDefine.JOINT_POINT_KIND)
			{
				csCalModel = new JointPoint();
			}
			else if (csModelKind == ToolDefine.DIVISION_POINT_KIND)
			{
				csCalModel = new DivisionPoint();
			}
			else if (csModelKind == ToolDefine.COMPARE_DATA_KIND)
			{
				csCalModel = new CompareData();
			}
			else if (csModelKind == ToolDefine.GENERATE_DATA_KIND)
			{
				csCalModel = new GenerateHydrologyData();
			}

			else
			{
			}
			
			return csCalModel;
		}

		/// <summary><para>method outline:</para>
		/// <para>要素内演算中データクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McCalInfo csCalModel = CreateCalInfo( csModelKind ) </para>
		/// </example>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>要素内演算中データクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCalInfo CreateCalInfo(HySObjectKind csModelKind)
		{
			McCalInfo csCalInfoData = null;

			if (csModelKind == ToolDefine.FILE_IN_KIND)
			{
				csCalInfoData = new FileInputBasicInfo();
			}
			else if (csModelKind == ToolDefine.FILE_IN_MATRIX_KIND)
			{
				csCalInfoData = new FileInputMatrixInfo();
			}
			else if (csModelKind == ToolDefine.FILE_OUT_KIND)
			{
				csCalInfoData = new FileOutputBasicInfo();
			}
			else if (csModelKind == ToolDefine.JOINT_POINT_KIND)
			{
				csCalInfoData = new JointPointInfo();
			}
			else if (csModelKind == ToolDefine.DIVISION_POINT_KIND)
			{
				csCalInfoData = new DivisionPointInfo();
			}
			else if (csModelKind == ToolDefine.COMPARE_DATA_KIND)
			{
				csCalInfoData = new CompareDataInfo();
			}
			else if (csModelKind == ToolDefine.GENERATE_DATA_KIND)
			{
				csCalInfoData = new GenerateHydrologyDataInfo();
			}
			else
			{
			}

			return csCalInfoData;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデルプロパティ表示／設定情報クラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McPropertyInfoRoot csPropertyInf = CreateModelPropertyInfo(csLibraryID, csModelKind ) </para>
		/// </example>
		/// <param name="csLibraryID">ライブラリー識別子</param>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>モデルプロパティ表示／設定情報クラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McCellModelPropertyIF CreateModelProperty(HySID csLibraryID, HySObjectKind csModelKind)
		{
			if (this.EqualFactory(csLibraryID) == false)
			{
				return null;
			}

			McCellModelPropertyInfo csRtnCellPrptyDt = null;

			if (csModelKind == ToolDefine.FILE_IN_KIND)
			{
				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);
				//csRtnCellPrptyDt.RemoveInfoType(McModelPropertyInfo.TIME_STEP);  // <-- モデルに⊿Ｔ設定は不要の為
                csRtnCellPrptyDt.RemoveInfoType(McPropertyInfo.TIME_STEP);  // <-- モデルに⊿Ｔ設定は不要の為

                csRtnCellPrptyDt.AddInfoType("m_csReadFileNameWithPath", "入力ファイル名（パス付き）", McDefine.ValKind.STRING);
				csRtnCellPrptyDt.SetInfo("m_csReadFileNameWithPath", "./ModelSample/FlowSample1.dat");

				csRtnCellPrptyDt.AddInfoType("m_dMarginTime", "計算時刻より先まで送信する時間(sec)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dMarginTime", 600);
				
				csRtnCellPrptyDt.AddInfoType("m_lOriginYear", "入力基準年(1970～)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOriginYear", 1970);
				csRtnCellPrptyDt.AddInfoType("m_lOriginMonth", "入力基準月(1～12)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOriginMonth", 1);
				csRtnCellPrptyDt.AddInfoType("m_lOriginDay", "入力基準日(1～31)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOriginDay", 1); 
				
				// 送信パターンの設定
				McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.IN_OUT_PATTERN_ANY,
					McTranInfoDefine.SINGLE_CELL_SERIAL,
					"ポイントセル出力") as McTranInfoPattern;
				{
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "AnyValue", HySDataKindDefine.ANY_VALUE, "Any");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddSendPattern(csTrnPtn);
			}
			else if (csModelKind == ToolDefine.FILE_IN_MATRIX_KIND)
			{
				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);
                //csRtnCellPrptyDt.RemoveInfoType(McModelPropertyInfo.TIME_STEP);  // <-- モデルに⊿Ｔ設定は不要の為
                csRtnCellPrptyDt.RemoveInfoType(McPropertyInfo.TIME_STEP);  // <-- モデルに⊿Ｔ設定は不要の為

                csRtnCellPrptyDt.AddInfoType("m_csReadFileNameWithPath", "入力ファイル名（パス付き）", McDefine.ValKind.STRING);
				csRtnCellPrptyDt.SetInfo("m_csReadFileNameWithPath", "./ModelSample/RainSample.dat");

				csRtnCellPrptyDt.AddInfoType("m_dMarginTime", "計算時刻より先まで送信する時間(sec)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dMarginTime", 3600);

				csRtnCellPrptyDt.AddInfoType("m_lOriginYear", "入力基準年(1970～)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOriginYear", 1970);
				csRtnCellPrptyDt.AddInfoType("m_lOriginMonth", "入力基準月(1～12)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOriginMonth", 1);
				csRtnCellPrptyDt.AddInfoType("m_lOriginDay", "入力基準日(1～31)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOriginDay", 1);

				
				// 送信パターンの設定
				McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.IN_OUT_PATTERN_2D,
					McTranInfoDefine.D2_CELL_SERIAL,
					"二次元セル形式出力") as McTranInfoPattern;
				{
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "AnyValue", HySDataKindDefine.ANY_VALUE, "Any");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddSendPattern(csTrnPtn);

				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(ToolDefine.IN_OUT_PATTERN_ANY, 
					McTranInfoDefine.SINGLE_CELL_SERIAL, "ポイントセルに変換して出力") as McTranInfoPattern;
				{
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1); 
					{
						csCellChara.SetDataKind(0, "AnyValue", HySDataKindDefine.ANY_VALUE, "Any"); 
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddSendPattern(csTrnPtn);
			}
			else if (csModelKind == ToolDefine.FILE_OUT_KIND)
			{   // HYDROLOGY形式ファイル出力データクラス

				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);
                //csRtnCellPrptyDt.RemoveInfoType(McModelPropertyInfo.TIME_STEP);  // <-- モデルに⊿Ｔ設定は不要の為
                csRtnCellPrptyDt.RemoveInfoType(McPropertyInfo.TIME_STEP);  // <-- モデルに⊿Ｔ設定は不要の為

                csRtnCellPrptyDt.AddInfoType("m_csOutFileNameWithPath", "出力ファイル名（*:セル一括出力  &t:時間を挿入）", McDefine.ValKind.STRING);
				csRtnCellPrptyDt.SetInfo("m_csOutFileNameWithPath", "./ModelSample/Output/FileOut(*)-&t.dat");

				csRtnCellPrptyDt.AddInfoType("m_lOutDataIdx", "ファイルを出力するセル番号（一括出力なら使用しない）", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOutDataIdx", 0);
				
				csRtnCellPrptyDt.AddInfoType("m_csOutTimeRange", "データ出力の時間間隔（単位：秒）（0にするとデータをすべて出力）", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_csOutTimeRange", 0);

				csRtnCellPrptyDt.AddInfoType("m_lOriginYear", "出力基準年(1970～)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOriginYear", 1970);
				csRtnCellPrptyDt.AddInfoType("m_lOriginMonth", "出力基準月(1～12)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOriginMonth", 1);
				csRtnCellPrptyDt.AddInfoType("m_lOriginDay", "出力基準日(1～31)", McDefine.ValKind.LONG);
				csRtnCellPrptyDt.SetInfo("m_lOriginDay", 1);

				// 受信パターンの設定
				McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.IN_OUT_PATTERN_ANY,
					McTranInfoDefine.SINGLE_CELL_SERIAL,
					"データ入力") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "AnyValue", HySDataKindDefine.ANY_VALUE, "Any");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);
			}
			
			else if (csModelKind == ToolDefine.JOINT_POINT_KIND)
			{
				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);

				csRtnCellPrptyDt.AddInfoType("m_dStepTime", "データを集合する間隔(sec)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dStepTime", 60.0);

				// 受信パターンの設定
				McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					new HySID("毎秒データ"),McTranInfoDefine.SINGLE_CELL_SERIAL, "毎秒データ") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1); 
					{
						csCellChara.SetDataKind(0, "毎秒データ", HySDataKindDefine.ANY_VALUE, "(N/sec)");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);

				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					new HySID("毎分データ"), McTranInfoDefine.SINGLE_CELL_SERIAL, "毎分データ") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "毎分データ", HySDataKindDefine.ANY_VALUE, "(N/min)");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);

				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					new HySID("毎時データ"), McTranInfoDefine.SINGLE_CELL_SERIAL, "毎時データ") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "毎時データ", HySDataKindDefine.ANY_VALUE, "(N/hour)");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);

				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					new HySID("毎日データ"), McTranInfoDefine.SINGLE_CELL_SERIAL, "毎日データ") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "毎日データ", HySDataKindDefine.ANY_VALUE, "(N/day)");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);
				

				// 送信パターンの設定
				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.IN_OUT_PATTERN_ANY,
					McTranInfoDefine.SINGLE_CELL_SERIAL,"通常出力") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(3);
					{
						csCellChara.SetDataKind(0, "期間平均出力", HySDataKindDefine.ANY_VALUE, "Any");
						csCellChara.SetDataKind(1, "期間合計出力", HySDataKindDefine.ANY_VALUE, "Any");
						csCellChara.SetDataKind(2, "期間の長さ", HySDataKindDefine.ANY_VALUE, "(sec)");

					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddSendPattern(csTrnPtn);
			}
			else if (csModelKind == ToolDefine.DIVISION_POINT_KIND)
			{
				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);
				csRtnCellPrptyDt.SetStepTime(60.0);

				csRtnCellPrptyDt.AddInfoType("m_dA", "分割係数：傾きA（Y = A*X + B）", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dA", 0.5);

				csRtnCellPrptyDt.AddInfoType("m_dB", "分割係数：切片B（Y = A*X + B）", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dB", 0.0);

				csRtnCellPrptyDt.AddInfoType("m_dYMax", "分割値：Yの上限値（0にすると上限なし）", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dYMax", 0.0);

				// 受信パターンの設定
				McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.IN_OUT_PATTERN_ANY,
					McTranInfoDefine.SINGLE_CELL_SERIAL, "入力値 (X)") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "AnyValue", HySDataKindDefine.ANY_VALUE, "Any");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);

				// 送信パターンの設定
				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.OUT_PATTERN_BOTTOM,
					McTranInfoDefine.SINGLE_CELL_SERIAL, "分割した側 （Y）") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "AnyValue", HySDataKindDefine.ANY_VALUE, "Any");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddSendPattern(csTrnPtn);

				// 送信パターンの設定
				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.OUT_PATTERN_SECOND,
					McTranInfoDefine.SINGLE_CELL_SERIAL, "分割を受けた側 （X - Y）") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "AnyValue", HySDataKindDefine.ANY_VALUE, "Any");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddSendPattern(csTrnPtn);

			}
			else if (csModelKind == ToolDefine.COMPARE_DATA_KIND)
			{
				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);

				// 受信パターンの設定
				McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.INPUT_1,
					McTranInfoDefine.SINGLE_CELL_SERIAL, "Input1") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "AnyValue", HySDataKindDefine.ANY_VALUE, "Any");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);

				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
				   ToolDefine.INPUT_2,
				   McTranInfoDefine.SINGLE_CELL_SERIAL, "Input2") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "AnyValue", HySDataKindDefine.ANY_VALUE, "Any");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddReceivePattern(csTrnPtn);

				// 送信パターンの設定
				csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.IN_OUT_PATTERN_ANY,
					McTranInfoDefine.SINGLE_CELL_SERIAL, "比較結果出力") as McTranInfoPattern;
				{
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(2);
					{
						csCellChara.SetDataKind(0, "差計算(Input1 - Input2)", HySDataKindDefine.ANY_VALUE, "Any");
						csCellChara.SetDataKind(1, "比率計算(Input1 / Input2)", HySDataKindDefine.ANY_VALUE, "Any");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddSendPattern(csTrnPtn);

			}
			else if (csModelKind == ToolDefine.GENERATE_DATA_KIND)
			{   // テスト用流量発生クラス
				csRtnCellPrptyDt = new McCellModelPropertyInfo(csLibraryID, csModelKind);

				csRtnCellPrptyDt.SetStepTime(72); // δT設定 秒

				csRtnCellPrptyDt.AddInfoType("m_dQbasic", "基礎流量", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dQbasic", 200.0);
				csRtnCellPrptyDt.AddInfoType("m_dQpeak", "ピーク流量", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dQpeak", 2000.0);
				csRtnCellPrptyDt.AddInfoType("m_dTpeak", "ピーク正規時間(単位: hr)", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dTpeak", 10.0);
				csRtnCellPrptyDt.AddInfoType("m_dCpeak", "係数c", McDefine.ValKind.DOUBLE);
				csRtnCellPrptyDt.SetInfo("m_dCpeak", 10.0);

				// 送信パターンの設定
				McTranInfoPattern csTrnPtn = csRtnCellPrptyDt.CreateTranInforPattern(
					ToolDefine.OUT_PATTERN_BOTTOM,
					McTranInfoDefine.SINGLE_CELL_SERIAL,
					"データ出力") as McTranInfoPattern;
				{
					// csTrnPtn.SetDim(1); // セルの配列（1次元：1個のみ）
					// セル内変数設定
					HySDataCharacteristicInCell csCellChara = csTrnPtn.CreateCellDataCharacteristic(1);
					{
						csCellChara.SetDataKind(0, "流量", HySDataKindDefine.QUANTITY_OF_WATER_FLOW, "m3/sec");
					}
					csTrnPtn.SetCellDataCharacteristic(csCellChara);
				}
				csRtnCellPrptyDt.AddSendPattern(csTrnPtn);

			}
			else 
			{
				// Do nothing
			}

			return csRtnCellPrptyDt;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデル初期化表示／設定情報生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McPropertyInfoRoot csInitialInf = CreateModelInitialInfo(csLibraryID, csModelKind ) </para>
		/// </example>
		/// <param name="csLibraryID">ライブラリー識別子</param>
		/// <param name="csModelKind">モデル種別識別子</param>
		/// <returns>初期情報クラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override McPropertyInfoRoot CreateModelInitialInfo(HySID csLibraryID, HySObjectKind csModelKind)
		{
			if (this.EqualFactory(csLibraryID) == false)
			{
				return null;
			}

			McInitialInfo csRtnDt = null;

			if (csModelKind == ToolDefine.FILE_IN_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);
			}
			else if (csModelKind == ToolDefine.FILE_IN_MATRIX_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);
			}
			else if (csModelKind == ToolDefine.FILE_OUT_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);
			}
			else if (csModelKind == ToolDefine.JOINT_POINT_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);
			}
			else if (csModelKind == ToolDefine.DIVISION_POINT_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);
			}
			else if (csModelKind == ToolDefine.COMPARE_DATA_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);
			}
			else if (csModelKind == ToolDefine.GENERATE_DATA_KIND)
			{
				csRtnDt = new McInitialInfo(csLibraryID, csModelKind);
			}
			else
			{
				// Do Nothing
			}

			return csRtnDt;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル情報提供</para>
		/// </summary>
		/// <example>
		/// <para>usage:</para>
		/// <para>HySDataLinkedList csModelInfoList = GetCalModelInfoList();</para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>モデル情報のリスト</returns>
		/// <exception cref="">無し</exception>
		/// <remarks>
		/// <para>remarks:</para>
		/// <para>モデル情報(McModelInfo)をリスト形式で返却する</para>
		/// </remarks>
		public override HySDataLinkedList GetCalModelInfoList()
		{
			McModelInfo csModelInfo = null;
			
			csModelInfo = new McModelInfo(
				   (HySID)this.GetFactoryID(),
				   McModelLibraryDefine.DIVISION_INPUT_MODEL,
				   McModelLibraryDefine.MODEL_CLASSIFICATION_INPUT_SPF_FILE,
				   ToolDefine.FILE_IN_KIND,
				   ToolDefine.FILE_IN_NAME);
            csModelInfo.SetVersionInf("Ver1.1 Nov. 20, 2010");
            csModelInfo.SetSummaryInf("京都大学水文・水資源学研究室形式のファイル入力モデル");
            csModelInfo.SetCreatorInf("京都大学 大学院 工学研究科 水文・水資源学研究室");
			csModelInfo.SetIconName("DefaultInput");
			m_csCalModelInforList.AddLast(csModelInfo);

			csModelInfo = new McModelInfo(
				   (HySID)this.GetFactoryID(),
				   McModelLibraryDefine.DIVISION_INPUT_MODEL,
				   McModelLibraryDefine.MODEL_CLASSIFICATION_INPUT_SPF_FILE,
				   ToolDefine.FILE_IN_MATRIX_KIND,
				   ToolDefine.FILE_IN_MATRIX_NAME);
            csModelInfo.SetVersionInf("Ver1.0 Nov. 20, 2010");
            csModelInfo.SetSummaryInf("京都大学水文・水資源学研究室形式の2次元ファイル入力モデル");
            csModelInfo.SetCreatorInf("京都大学 大学院 工学研究科 水文・水資源学研究室");
			csModelInfo.SetIconName("DefaultInput");
			m_csCalModelInforList.AddLast(csModelInfo);

			csModelInfo = new McModelInfo(
					(HySID)this.GetFactoryID(),
					McModelLibraryDefine.DIVISION_OUTPUT_MODEL,
					McModelLibraryDefine.MODEL_CLASSIFICATION_OUTPUT_SPF_FILE,
					ToolDefine.FILE_OUT_KIND,
					ToolDefine.FILE_OUT_NAME);
            csModelInfo.SetVersionInf("Ver1.0 Nov. 20, 2010");
            csModelInfo.SetSummaryInf("京都大学水文・水資源学研究室形式のファイル出力モデル");
            csModelInfo.SetCreatorInf("京都大学 大学院 工学研究科 水文・水資源学研究室");
			csModelInfo.SetIconName("DefaultOutput");
			m_csCalModelInforList.AddLast(csModelInfo);

			csModelInfo = new McModelInfo(
				(HySID)this.GetFactoryID(),
				McModelLibraryDefine.DIVISION_CALCULATION_MODEL,
				// McModelLibraryDefine.MODEL_CLASSIFICATION_CAL_HYDROLOGY,
                new HySObjectKind("CAL_MODELS"),
				ToolDefine.JOINT_POINT_KIND,
				ToolDefine.JOINT_POINT_NAME);
            csModelInfo.SetVersionInf("Ver1.1 Nov. 20, 2010");
			csModelInfo.SetSummaryInf("複数の流れを一つにまとめる際に用いる合流点モデル");
            csModelInfo.SetCreatorInf("京都大学 大学院 工学研究科 水文・水資源学研究室");
            csModelInfo.SetIconName("DefaultModel");
			m_csCalModelInforList.AddLast(csModelInfo);

			csModelInfo = new McModelInfo(
				(HySID)this.GetFactoryID(),
				McModelLibraryDefine.DIVISION_CALCULATION_MODEL,
				// McModelLibraryDefine.MODEL_CLASSIFICATION_CAL_HYDROLOGY,
                new HySObjectKind("CAL_MODELS"),
				ToolDefine.DIVISION_POINT_KIND,
				ToolDefine.DIVISION_POINT_NAME);
            csModelInfo.SetVersionInf("Ver1.2 Dec. 20, 2010");
			csModelInfo.SetSummaryInf("分流モデル");
			// パラメータAとBを指定する Y=AX＋Bの線形関数により、データの分割を行う．
            // 分流後の流量は負にならないようにしている．
            csModelInfo.SetCreatorInf("京都大学大学院 工学研究科 水文・水資源学研究室");
            csModelInfo.SetIconName("DefaultModel");
			m_csCalModelInforList.AddLast(csModelInfo);

			csModelInfo = new McModelInfo(
					(HySID)this.GetFactoryID(),
					McModelLibraryDefine.DIVISION_CALCULATION_MODEL,
					//McModelLibraryDefine.MODEL_CLASSIFICATION_CAL_HYDROLOGY,
                    new HySObjectKind("CAL_MODELS"),
					ToolDefine.COMPARE_DATA_KIND,
					ToolDefine.COMPARE_DATA_NAME);
            csModelInfo.SetVersionInf("Ver1.1 Nov. 20, 2010");
			csModelInfo.SetSummaryInf("二つのデータを比較するモデル");
            csModelInfo.SetCreatorInf("京都大学 大学院 工学研究科 水文・水資源学研究室");
            csModelInfo.SetIconName("DefaultModel");
			m_csCalModelInforList.AddLast(csModelInfo);

			csModelInfo = new McModelInfo(
					(HySID)this.GetFactoryID(),
					McModelLibraryDefine.DIVISION_CALCULATION_MODEL,
					//McModelLibraryDefine.MODEL_CLASSIFICATION_CAL_HYDROLOGY,
                    new HySObjectKind("CAL_MODELS"),
					ToolDefine.GENERATE_DATA_KIND,
					ToolDefine.GENERATE_DATA_NAME);
            csModelInfo.SetVersionInf("Ver1.3 Nov. 20, 2010");
			csModelInfo.SetSummaryInf("テスト用流量を発生する際に用いるモデル");
			csModelInfo.SetCreatorInf("京都大学 大学院 工学研究科 水文・水資源学研究室");
            csModelInfo.SetIconName("DefaultModel");
			m_csCalModelInforList.AddLast(csModelInfo);

			return m_csCalModelInforList;
		}
	}
}

namespace CommonMP.HYMCO._SYSTEM_
{
	/// <summary><para>class outline:</para>
	/// <para>KyotoUniv.Hydrologyツール系モデルファクトリクラス</para>
	/// </summary>
	public class HYMCO_MODEL_FACTORY_CREATOR_IMPLE : HYMCO_MODEL_FACTORY_CREATOR
	{
		/// <summary><para>method outline:</para>
		/// <para>コンストラクター</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> HYMCO_MODEL_FACTORY_CREATOR_IMPLE csFactCrt = new HYMCO_MODEL_FACTORY_CREATOR_IMPLE( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>生成インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public HYMCO_MODEL_FACTORY_CREATOR_IMPLE()
		{
		}
		/// <summary><para>method outline:</para>
		/// <para>演算モデルファクトリクラス生成</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McModelFactory csCalModelFactory = CreateFactory( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>演算モデルファクトリクラス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public CommonMP.HYMCO.Interface.Model.McModelFactory CreateFactory()
		{
			return new KyotoUnivEngHywr.ToolFactory();
		}
	}
}