﻿using System;
using System.Collections.Generic;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreImpl.Model;
using CommonMP.HYMCO.CoreImpl.Tool;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>OHyMoSデータファイル入力モデルクラス</para>
	/// </summary>
	public class FileInputBasic : McBasicFileInCtl
	{
		/// <summary> 内部変数 </summary>
		FileInputBasicInfo mInf = new FileInputBasicInfo();

		/// <summary><para>method outline:</para>
		/// <para>出力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SendConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>送信端子に設定されている伝送データが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool SendConnectionCheck(ref McStructErrorInfo csErrorInf, long lOutputDataNum, McSendCellDataIF[] csOutputCellData)
		{
			bool bRtn = true;
			if (lOutputDataNum == 0)
			{
				// 警告（エラーではない）
				csErrorInf.AddCheckWarningData(this.GetID(), ToolDefine.FILE_IN_KIND, "No send port.");
			}
			for (long lP = 0; lP < lOutputDataNum; lP++)
			{
				if (csOutputCellData[lP].GetSendPatternID().Equals(ToolDefine.IN_OUT_PATTERN_ANY) == false)
				{
					// 予期しない接続が行われている場合
					csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.FILE_IN_KIND,
						"Unexpected send data type. (Send to " + csOutputCellData[lP].GetLowerElementID().ToString() + ")");
					bRtn = false;
				}
			}
			return bRtn;
		}

		
		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>Initialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool Initialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "<<Init>>");
		 
			HySTime csTime = new HySTime(0.0);
			InputBasicRecord csRcd;
			long lRtn = 0;

			mInf.m_csLastOutTime = new HySTime(0.0);

			// データを格納するLinkedListを宣言する
			mInf.m_csDataList = new LinkedList<InputBasicRecord>();

			// データの入力基準時間 m_csOriginTimeをパラメタから生成する
			mInf.m_csOriginTime = HySCalendar.CreateTime(mInf.m_lOriginYear, mInf.m_lOriginMonth, mInf.m_lOriginDay, 0, 0, 0);

			// 入力ファイルを開く
			HySFile m_csInFile = new HySFile(mInf.m_csReadFileNameWithPath);
			m_csInFile.Open(HySFile.OPEN_MODE.OPEN, HySFile.READ_WRITE_MODE.READ, HySFile.DIRECTORY_MODE.MK_DIR);

			String sLine = "";
			while (true)
			{
				lRtn = m_csInFile.ReadText(ref sLine);
				if (lRtn == 0 || lRtn == -1)
				{
					// ファイルの終点に達した
					break;
				}

				// 行頭が「#」から始まる行はコメントなので読み込まない
				if (sLine[0] != '#')
				{
					// 読み取った一行分の文字列を空白とカンマで区切る
					String[] str = sLine.Split(new Char[] { ' ', ',' }, StringSplitOptions.RemoveEmptyEntries);

					// 配列長が2でない行が見つかった場合、入力ファイルを閉じてエラーを返す
					if (str.Length != 2)
					{
						m_csInFile.Close();		
						return false;
					}

					// データ格納クラスの生成
					csRcd = new InputBasicRecord();  
					
					// 時間とデータをを設定する
					csRcd.m_csTime = new HySTime(double.Parse(str[0]) + mInf.m_csOriginTime.TotalSec());
					csRcd.m_dDT = double.Parse(str[1]);

					// データリストに追加する
					mInf.m_csDataList.AddLast(csRcd);
				}
			}
			
			// 入力ファイルを閉じる
			m_csInFile.Close();		
			
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>OnlineInitialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>bool</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool OnlineInitialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデル演算結果を外部のエレメントに対して公開する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = DataFusion( lOutputDataNum, ref csOutputCellData)</para>
		/// </example>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long DataFusion(long lOutputDataNum, ref McSendCellDataIF[] csOutputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "<<Start>>");
			
			HySCellData[] csSndCellData = null;
			LinkedListNode<InputBasicRecord> csDtNode;
			InputBasicRecord csData = null;

			while (true)
			{
				if (mInf.m_csDataList.Count <= 0)
				{
					break;
				}
				
				// 保管情報を1レコードづつ取り出す
				csDtNode = mInf.m_csDataList.First;
				csData = csDtNode.Value;

				if (csData.m_csTime.TotalSec() <= m_csSimTime.TotalSec() + mInf.m_dMarginTime || mInf.m_csLastOutTime.TotalSec() < m_csSimTime.TotalSec())
				{
					for (long lP = 0; lP < lOutputDataNum; lP++)
					{
						csOutputCellData[lP].SetCurrentTime(csData.m_csTime);
						csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
						csSndCellData[0].m_dData[0] = csData.m_dDT;
						m_OutputCellData[lP].SendData();
					}

					mInf.m_csLastOutTime = csData.m_csTime.Clone();
					mInf.m_csDataList.RemoveFirst();
				}
				else
				{
					break;
				}
			}
			
			return 0;
		}

		/// <summary><para>method outline:</para>
		/// <para>プロパティ情報を設定する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>SetProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "<<Init>>");
			McModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McModelPropertyInfo;
			if (csPrptyInfo != null)
			{
				csPrptyInfo.GetInfo("m_csReadFileNameWithPath", ref this.mInf.m_csReadFileNameWithPath);
				csPrptyInfo.GetInfo("m_dMarginTime", ref this.mInf.m_dMarginTime);
				csPrptyInfo.GetInfo("m_lOriginYear", ref this.mInf.m_lOriginYear);
				csPrptyInfo.GetInfo("m_lOriginMonth", ref this.mInf.m_lOriginMonth);
				csPrptyInfo.GetInfo("m_lOriginDay", ref this.mInf.m_lOriginDay);
			}
			return true;
		}

		/// <summary><para>method outline:</para>
		/// <para>演算実行中断中のプロパティ等情報設定</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SetOnlineProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>演算中断中にプロパティ、初期値等を変更して動作させ場合等に使用する</para>
		/// </remarks>
		public override bool SetOnlineProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			return true;
		}
 

	}
}
