﻿using System;
using System.Collections.Generic;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreImpl.Model;
using CommonMP.HYMCO.CoreImpl.Tool;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>OHyMoS 2次元データファイル入力モデルクラス</para>
	/// </summary>
	public class FileInputMatrix : McBasicFileInCtl
	{
		/// <summary> 内部変数 </summary>
		FileInputMatrixInfo mInf = new FileInputMatrixInfo();

				/// <summary><para>method outline:</para>
		/// <para>出力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SendConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>送信端子に設定されている伝送データが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool SendConnectionCheck(ref McStructErrorInfo csErrorInf, long lOutputDataNum, McSendCellDataIF[] csOutputCellData)
		{
			bool bRtn = true;
			if (lOutputDataNum == 0)
			{
				// 警告（エラーではない）
				csErrorInf.AddCheckWarningData(this.GetID(), ToolDefine.FILE_IN_MATRIX_KIND, "No send port.");
			}

			for (long lP = 0; lP < lOutputDataNum; lP++)
			{
				if (csOutputCellData[lP].GetSendPatternID().Equals(ToolDefine.IN_OUT_PATTERN_ANY))
				{
				}
				else if (csOutputCellData[lP].GetSendPatternID().Equals(ToolDefine.IN_OUT_PATTERN_2D))
				{

				}
				else
				{
					// 予期しない接続が行われている場合
					csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.FILE_IN_MATRIX_KIND,
						"Unexpected send data type. (Send to " + csOutputCellData[lP].GetLowerElementID().ToString() + ")");
					bRtn = false;
				}
			}
			return bRtn;
		}


		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>Initialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool Initialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		// protected override bool Initialize(ref McPropertyInfoRoot csInitialData)
		{

			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "<<Init>>");
			// 此処では、計算速度アップの為、最初に全ての情報を読んでおく
			HySTime csTime = new HySTime(0.0);
			Input2DRecord csRcd;
			long lRtn = 0;

			mInf.m_csLastOutTime = new HySTime(0.0);
			mInf.m_csDataList = new LinkedList<Input2DRecord>();
			// 入力基準時間 m_csOriginTimeの生成
			mInf.m_csOriginTime = HySCalendar.CreateTime(mInf.m_lOriginYear, mInf.m_lOriginMonth, mInf.m_lOriginDay, 0, 0, 0);
			try
			{
				// 入力ファイル
				HySFile m_csInFile = new HySFile(mInf.m_csReadFileNameWithPath);
				m_csInFile.Open(HySFile.OPEN_MODE.OPEN, HySFile.READ_WRITE_MODE.READ, HySFile.DIRECTORY_MODE.MK_DIR);

				String sLine = "";
				while (true)
				{
					lRtn = m_csInFile.ReadText(ref sLine);
					if (lRtn == 0 || lRtn == -1)
					{
						// ファイルの終点に達した
						break;
					}

					// コメント行である場合は飛ばす
					if (sLine[0] != '#')
					{
						// まず最初に時間と横の数　縦の数が読み込まれる
						String[] str = sLine.Split(new Char[] { ' ', ':' }, StringSplitOptions.RemoveEmptyEntries);
						
						long yoko = long.Parse(str[1]);
						long tate = long.Parse(str[2]);

						csRcd = new Input2DRecord((int)yoko, (int)tate);
						csRcd.m_csTime = new HySTime(double.Parse(str[0]) + mInf.m_csOriginTime.TotalSec());

						// 縦の回数分繰り返す
						for (int i = 0; i < tate; i++)
						{
							// 横の回数分代入する
							m_csInFile.ReadText(ref sLine);
							str = sLine.Split(new Char[] { ' ', ':' }, StringSplitOptions.RemoveEmptyEntries);

							if (str.Length == yoko)
							{
								for (int j = 0; j < yoko; j++)
								{
									csRcd.m_dDT[j, i] = double.Parse(str[j]);

								}
							}
							else
							{
								// エラー
							}

						}

						mInf.m_csDataList.AddLast(csRcd);
						// m_csOutFileInfo.m_csLastINTime.SetTime(csRcd.m_csTime);

						// m_csInFileInfo.m_csReadData.AddData(csTime, csValue);
					}

				}

				m_csInFile.Close();
			}
			catch (Exception e)
			{
				McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "IO Error FileName:" + mInf.m_csReadFileNameWithPath + "," + e.Message);
				return false;
			}
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>OnlineInitialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>bool</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool OnlineInitialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデル演算結果を外部のエレメントに対して公開する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = DataFusion( lOutputDataNum, ref csOutputCellData)</para>
		/// </example>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long DataFusion(long lOutputDataNum, ref McSendCellDataIF[] csOutputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "<<Start>>");

			HySCellData[] csSndCellData = null;
			HySCellData[,] csSndCellData2D = null;

			LinkedListNode<Input2DRecord> csDtNode;
			Input2DRecord csData = null;

			while (true)
			{
				if (mInf.m_csDataList.Count <= 0)
				{
					break;
				}

				// 保管情報を1レコードづつ取り出す
				csDtNode = mInf.m_csDataList.First;
				csData = csDtNode.Value;

				if (csData.m_csTime.TotalSec() <= m_csSimTime.TotalSec() + mInf.m_dMarginTime ||  mInf.m_csLastOutTime.TotalSec() < m_csSimTime.TotalSec())
				{
					for (long lP = 0; lP < lOutputDataNum; lP++)
					{
						if (ToolDefine.IN_OUT_PATTERN_ANY.Equals(csOutputCellData[lP].GetSendPatternID()) == true)
						{
							double calcdata = 0.0;
							for (int x = 0; x < csData.dx; x++)
							{
								for (int y = 0; y < csData.dy; y++)
								{
									calcdata += csData.m_dDT[x, y];
								}
							}
							calcdata = calcdata / (csData.dx * csData.dy);

							csOutputCellData[lP].SetCurrentTime(csData.m_csTime);
							csSndCellData = csOutputCellData[lP].PrepareSendCellD1();

							csSndCellData[0].m_dData[0] = calcdata;
							m_OutputCellData[lP].SendData();
						}
						else
						{
							// csOutputCellData[lP].SetDim(csData.dx, csData.dy);
							csOutputCellData[lP].SetCurrentTime(csData.m_csTime);
							csSndCellData2D = csOutputCellData[lP].PrepareSendCellD2();
							for (int x = 0; x < csData.dx; x++)
							{
								for (int y = 0; y < csData.dy; y++)
								{
									csSndCellData2D[x, y].m_dData[0] = csData.m_dDT[x, y];
								}
							}
							m_OutputCellData[lP].SendData();

						}
					}
					mInf.m_csLastOutTime = csData.m_csTime.Clone();

					mInf.m_csDataList.RemoveFirst();
				}
				else // 次のデータが未来のデータになるまで続ける
				{
					break;
				}
			}

			return 0;
		}

		/// <summary><para>method outline:</para>
		/// <para>プロパティ情報を設定する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>SetProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "<<Init>>");
			McModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McModelPropertyInfo;
			if (csPrptyInfo != null)
			{   // プロパティ情報あり

				// 出力ファイルの設定
				csPrptyInfo.GetInfo("m_csReadFileNameWithPath", ref this.mInf.m_csReadFileNameWithPath);

				csPrptyInfo.GetInfo("m_dMarginTime", ref this.mInf.m_dMarginTime);

				// csPropInf.GetInfo("m_lOutDataIdx", ref this.m_csInFileInfo.m_lOutDataIdx);
				csPrptyInfo.GetInfo("m_lOriginYear", ref this.mInf.m_lOriginYear);
				csPrptyInfo.GetInfo("m_lOriginMonth", ref this.mInf.m_lOriginMonth);
				csPrptyInfo.GetInfo("m_lOriginDay", ref this.mInf.m_lOriginDay);
				
				
				// 演算ステップ時刻設定
				// this.m_csDltTime = new HySTime(60.0);// csPrptyInfo.GetStepTime());

				// シーケンシャルアクセス時間間隔を取得
				// long lDt = 0;
				// csPropInf.GetInfo("m_csOutTimeLag", ref lDt);
				// m_csOutFileInfo.m_csOutTimeLag = new HySTime(lDt * 60.0);

				// データ出力の時間間隔を取得
				// long lSec = 0;
				// csPropInf.GetInfo("m_csOutTimeRange", ref lSec);
				// 0が指定されていた場合は1にする
				// if (lSec == 0) lSec = 1;
				// m_csOutFileInfo.m_csOutTimeRange = new HySTime(lSec);
			}
			return true;
		}

		/// <summary><para>method outline:</para>
		/// <para>演算実行中断中のプロパティ等情報設定</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SetOnlineProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>演算中断中にプロパティ、初期値等を変更して動作させ場合等に使用する</para>
		/// </remarks>
		public override bool SetOnlineProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			return true;
		}
 

	}
}
