﻿using System;
using System.Collections.Generic;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;
using CommonMP.HYSSOP.Interface.HSData;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>OHyMoSデータファイル出力モデルクラス</para>
	/// </summary>
	[Serializable]
	public class FileOutputBasicInfo : McCalInfo
	{
		/// <summary> 出力ファイルの基準名称 </summary>
		public String m_csOutFileNameWithPath;

		/// <summary> 全セル数</summary>
		public long m_lAllCellNum;
		
		/// <summary> 出力ファイル名配列</summary>
		public String[] m_csAllOutFileName;

		/// <summary> 出力ファイルクラス</summary>
		public HySFile[] m_csAllOutFile;

		/// <summary> データ出力間隔 </summary>
		public HySTime m_csOutTimeRange;

		/// <summary> 出力時の基準年月日（パラメータより取得）</summary>
		public HySTime m_csOriginTime;

		/// <summary> 出力基準　年・月・日 </summary>
		public long m_lOriginYear, m_lOriginMonth, m_lOriginDay;


		/// <summary> 出力開始時 </summary>
		public HySTime m_csStartTime;
		
		/// <summary> 最終入力時刻 </summary>
		public HySTime m_csLastINTime;
		
		/// <summary> 最終出力時刻 </summary>
		public HySTime m_csLastOutTime;

		/// <summary> データ一時保管 </summary>
		public LinkedList<OutputBasicRecord> m_csDataList;

		/// <summary> ファイルを出力するセル番号（一括出力なら使用しない） </summary>
		public long m_lOutDataIdx;

        // m-suto 2015/12/25 start
        /// <summary> セル内の位置情報（一括出力なら使用しない） </summary>
        public long m_lOutDataIdxInCell;
        // m-suto 2015/12/25 end

		/// <summary><para>method outline:</para>
		/// <para>自己複製</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> Clone( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>同一内容で　別インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>収束演算時、特定の時刻に状態を戻す場合等に使用する</para>
		/// </remarks>
		public override McCalInfo Clone()
		{
			FileOutputBasicInfo csRtn = new FileOutputBasicInfo();
			csRtn.CopyInfo(this);
			return csRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>引数で与えられた情報を自分にコピーを行う</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> bool bRtn = CopyInfo(csOrgInfo) </para>
		/// </example>
		/// <param name="csOrgInfo">コピー元情報</param>
		/// <returns> bool true :正常 , false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool CopyInfo(McCalInfo csOrgInfo)
		{
			FileOutputBasicInfo csOrgDt = (FileOutputBasicInfo)csOrgInfo;

			this.m_csOutFileNameWithPath = csOrgDt.m_csOutFileNameWithPath;
			this.m_csLastINTime = csOrgDt.m_csLastINTime.Clone();
			this.m_csLastOutTime = csOrgDt.m_csLastOutTime.Clone();
			this.m_csDataList = csOrgDt.m_csDataList;
			this.m_lOutDataIdx = csOrgDt.m_lOutDataIdx;
			this.m_csStartTime = csOrgDt.m_csStartTime;

			this.m_csOutTimeRange = csOrgDt.m_csOutTimeRange;
			this.m_csOriginTime = csOrgDt.m_csOriginTime;
			this.m_lOriginYear = csOrgDt.m_lOriginYear;
			this.m_lOriginMonth = csOrgDt.m_lOriginMonth;
			this.m_lOriginDay = csOrgDt.m_lOriginDay;

			this.m_lAllCellNum = csOrgDt.m_lAllCellNum;

			this.m_csAllOutFile = new HySFile[this.m_lAllCellNum];
			this.m_csAllOutFileName = new String[this.m_lAllCellNum];
			for (long lnum = 0; lnum < this.m_lAllCellNum; lnum++)
			{
				this.m_csAllOutFileName[lnum] = (String)(csOrgDt.m_csAllOutFileName[lnum].Clone());
				this.m_csAllOutFile[lnum] = new HySFile(this.m_csAllOutFileName[lnum]);
			}


			return true;
		}
	}
	/// <summary><para>class outline:</para>
	/// <para>出力ファイルの1タイム分情報を記録するクラス</para>
	/// </summary>
	/// <remarks><para>history:</para>
	/// <para>[CommonMP][ver 1.0.0][2008/10/01][新規作成]</para>
	/// <para>remarks</para>
	/// <para>演算中の状態、初期条件等を保持するデータクラス</para>
	/// </remarks>
	[Serializable]
	public class OutputBasicRecord : HySDataRoot
	{
		/// <summary> 最終出力時刻 </summary>
		public HySTime m_csTime;
		/// <summary> データ一時保管 </summary>
		public double[] m_dDT;

		/// <summary><para>method outline:</para>
		/// <para>コンストラクター</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> McTestOutRecord csRd = McTestOutRecord( lDim ) </para>
		/// </example>
		/// <param name="lDim">1レコードのデータ数</param>
		/// <returns>生成されたインスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public OutputBasicRecord(long lDim)
		{
			m_dDT = new double[lDim];
		}
	}
}