﻿using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.Interface.HSData;
using CommonMP.HYSSOP.Interface.HSViewer;
using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;

using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.Interface.Controller;


using CommonMP.HYMCO.CoreImpl.Data;

namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>テスト用流量発生モデル内演算データクラス</para>
	/// </summary>
	/// <remarks><para>history:</para>
	/// <para>[CommonMP][ver 1.0.0][2008/10/01][新規作成]</para>
	/// <para>remarks</para>
	/// <para>演算中の状態、初期条件等を保持するデータクラス</para>
	/// </remarks>
	[Serializable]
	class JointPointInfo : McCalInfo
	{

		/*パラメタ*/

		/// <summary> パラメタ：ステップ時間 </summary>
		public double m_dStepTime = 0.0;

		/*計算変数*/

		/// <summary> データを書きこんだ最後の時間</summary>
		public double m_dLastDataTime;

		/// <summary> ステップ内データの総計 </summary>
		public double m_dStepTotal = 0.0;

		/// <summary> ステップ内データの総計を出力するための変数 </summary>
		public double m_dStepOutput = 0.0;

		/// <summary> ステップ内で何回のCalculateを行ったか </summary>
		public long m_lStepCount = 0;

		/// <summary> データフュージョンを行うフラッグ </summary>
		public bool m_bFusion = false;


		/// <summary> データ接続から決まる単位情報 
		/// データの時間単位を判別し、出力する 0:秒データ 1:分データ 2:時データ 3:日データ </summary>
		public long m_lDataKind = 0;

		/// <summary> データの時間単位を調整する係数 1,60,60*60,60*60*24のいずれか</summary>
		public double[] m_dK;


		/// <summary><para>method outline:</para>
		/// <para>自己複製</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> Clone( ) </para>
		/// </example>
		/// <param name="">無し</param>
		/// <returns>同一内容で　別インスタンス</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>収束演算時、特定の時刻に状態を戻す場合等に使用する</para>
		/// </remarks>
		public override McCalInfo Clone()
		{
			JointPointInfo csRtn = new JointPointInfo();
			csRtn.CopyInfo(this);
			return csRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>引数で与えられた情報を自分にコピーを行う</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para> bool bRtn = CopyInfo(csOrgInfo) </para>
		/// </example>
		/// <param name="csOrgInfo">コピー元情報</param>
		/// <returns> bool true :正常 , false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool CopyInfo(McCalInfo csOrgInfo)
		{
			JointPointInfo csOrgDt = (JointPointInfo)csOrgInfo;
			
			this.m_dStepTime = csOrgDt.m_dStepTime;
			
			this.m_bFusion = csOrgDt.m_bFusion;
			this.m_dLastDataTime = csOrgDt.m_dLastDataTime;
			this.m_dStepTotal = csOrgDt.m_dStepTotal;
			this.m_lStepCount = csOrgDt.m_lStepCount;
			this.m_dStepOutput = csOrgDt.m_dStepOutput;

			this.m_lDataKind = csOrgDt.m_lDataKind;

			for (int i = 0; i < 4; i++)
			{
				this.m_dK[i] = csOrgDt.m_dK[i];
			}
			return true;
		}

	}
}
