﻿using System;
using System.Collections.Generic;
using System.Text;

using CommonMP.HYSSOP.CoreImpl;
using CommonMP.HYSSOP.CoreImpl.HSData;
using CommonMP.HYSSOP.CoreImpl.HSTools;


using CommonMP.HYMCO.Interface.Model;
using CommonMP.HYMCO.Interface.Data;
using CommonMP.HYMCO.CoreImpl.Data;
using CommonMP.HYMCO.CoreImpl.Tool;



namespace KyotoUnivEngHywr
{
	/// <summary><para>class outline:</para>
	/// <para>テスト用流量発生クラス</para>
	/// </summary>
	/// <remarks><para>history:</para>
	/// <para>[CommonMP][ver 1.0.0][2008/10/01][新規作成]</para>
	/// </remarks>
	public class CompareData : McForecastModelBase
	{
		/// <summary>流域計算情報</summary>
		CompareDataInfo mInf = null;
		
		// =========================
		// 演算実行前処理関連メソッド
		// =========================

		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>Initialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool Initialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Initialize", "<<Init>>");

			mInf.m_data = new double[2];

			Calculate(lInputDataNum, ref csInputCellData);

			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>モデルを初期化する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>OnlineInitialize(csInitialData)</para>
		/// </example>
		/// <param name="csInitialData">初期化設定情報</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>bool</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override bool OnlineInitialize(ref McPropertyInfoRoot csInitialData, long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			// Do Nothing
			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>入力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = ReceiveConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>受信するデータが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool ReceiveConnectionCheck(ref McStructErrorInfo csErrorInf, long lInputDataNum, McReceiveCellDataIF[] csInputCellData)
		{
			bool bRtn = true;
			bool bInput1 = false;
			bool bInput2 = false;
			if (lInputDataNum == 0)
			{
				// 入力が無い場合 エラー
				csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.COMPARE_DATA_KIND, "No receive data.");
				bRtn = false;
			}
			else if (lInputDataNum > 0)
			{
				for (long lP = 0; lP < lInputDataNum; lP++)
				{
					if (csInputCellData[lP].GetReceivePatternID().Equals(ToolDefine.INPUT_1) == true)
					{
						if (bInput1 == false) 
						{
							bInput1 = true;
						}
						else
						{
							// エラー
							csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.COMPARE_DATA_KIND,
									"[Input1] receive connects are too many. (Received from " + csInputCellData[lP].GetUpperElementID().ToString() + ")");
							bRtn = false;
						}
					}
					else if (csInputCellData[lP].GetReceivePatternID().Equals(ToolDefine.INPUT_2) == true)
					{
						if (bInput2 == false)
						{
							bInput2 = true;
						}
						else
						{
							// エラー
							csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.COMPARE_DATA_KIND,
									"[Input2] receive connects are too many. (Received from " + csInputCellData[lP].GetUpperElementID().ToString() + ")");
							bRtn = false;
						}
					}
					else
					{
						// エラー
						csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.DIVISION_POINT_KIND,
								"Unexpected receive data type. (Received from " + csInputCellData[lP].GetUpperElementID().ToString() + ")");
						bRtn = false;
					}
				}
			}
			return bRtn;
		}
		/// <summary><para>method outline:</para>
		/// <para>出力側の接続情報チェック</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SendConnectionCheck(ref csErrorInf)</para>
		/// </example>
		/// <param name="csErrorInf">エラー出力</param>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=true:正常、=false:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>送信端子に設定されている伝送データが自モデルが期待している情報か否かをチェックする</para>
		/// </remarks>
		protected override bool SendConnectionCheck(ref McStructErrorInfo csErrorInf, long lOutputDataNum, McSendCellDataIF[] csOutputCellData)
		{
			bool bRtn = true;

			if (lOutputDataNum == 0)
			{
				// 警告（エラーではない）
				csErrorInf.AddCheckWarningData(this.GetID(), ToolDefine.DIVISION_POINT_KIND, "No send port.");
			}
			for (long lP = 0; lP < lOutputDataNum; lP++)
			{
				if (csOutputCellData[lP].GetSendPatternID().Equals(ToolDefine.IN_OUT_PATTERN_ANY))
				{
				}
				else
				{
					// エラー
					csErrorInf.AddCheckErrorData(this.GetID(), ToolDefine.DIVISION_POINT_KIND,
							"Unexpected send data type. (Send to " + csOutputCellData[lP].GetLowerElementID().ToString() + ")");
					bRtn = false;
				}
			}
			return bRtn;
		}

		// =======================
		// 演算実行処理関連メソッド
		// =======================

		/// <summary><para>method outline:</para>
		/// <para>モデル演算</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = Calculate(ref csInputCellData)</para>
		/// </example>
		/// <param name="lInputDataNum">入力情報数</param>
		/// <param name="csInputCellData">演算に必要な入力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long Calculate(long lInputDataNum, ref McReceiveCellDataIF[] csInputCellData)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "Calculate", "<<Start>>");

			HySCellData csCell = null;
			double d1 = 0.0;
			double d2 = 0.0;

			for (long lP = 0; lP < lInputDataNum; lP++)
			{
				csCell = csInputCellData[lP].GetInterpolatedCell(0);
				if (csInputCellData[lP].GetReceivePatternID().Equals(ToolDefine.INPUT_1) == true)
				{
					d1 = csInputCellData[lP].Data(csCell, 0);
				}
				else
				{
					d2 = csInputCellData[lP].Data(csCell, 0);
				}
			}

			mInf.m_data[0] = d1 - d2;
			if (d2 != 0.0)
			{
				mInf.m_data[1] = d1 / d2;
			}
			else
			{
				mInf.m_data[1] = d1 / (d2 - 0.0000001);
			}
			
			return 0;
		}

		/// <summary><para>method outline:</para>
		/// <para>モデル演算結果を外部のエレメントに対して公開する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>long lRtn = DataFusion( lOutputDataNum, ref csOutputCellData)</para>
		/// </example>
		/// <param name="lOutputDataNum">出力情報数</param>
		/// <param name="csOutputCellData">出力情報配列</param>
		/// <returns>=0:正常 -1:異常</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		protected override long DataFusion(long lOutputDataNum, ref McSendCellDataIF[] csOutputCellData)
		{
			HySCellData[] csSndCellData = null;

			// 期間出力時間がきたので期間出力を行う
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "DataFusion", "<<Start>>");

			for (long lP = 0; lP < m_lOutputDataNum; lP++)
			{
				csSndCellData = csOutputCellData[lP].PrepareSendCellD1();
				csSndCellData[0].m_dData[0] = mInf.m_data[0];
				csSndCellData[0].m_dData[1] = mInf.m_data[1];
			}
			return 0;
		}

		// ====================
		// その他必要なメソッド
		// ====================

		/// <summary><para>method outline:</para>
		/// <para>プロパティ情報を設定する</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>SetProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">セル型プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>無し</para>
		/// </remarks>
		public override bool SetProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			McLog.DebugOut(m_csSimTime, GetID(), GetType().ToString(), "SetProperty", "<<Init>>");
			// 使用しやすいようにキャストしておく
			mInf = (CompareDataInfo)m_csCalInfo;
			
			McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;
			if (csPrptyInfo != null)
			{
				this.m_csDltTime = new HySTime(csPrptyInfo.GetStepTime());
			}

			return true;
		}
		/// <summary><para>method outline:</para>
		/// <para>演算実行中断中のプロパティ等情報設定</para>
		/// </summary>
		/// <example><para>usage:</para>
		/// <para>bool bRtn = SetOnlineProperty(csCellMdlPropertyInfo)</para>
		/// </example>
		/// <param name="csCellMdlPropertyInfo">プロパティ情報</param>
		/// <returns>無し</returns>
		/// <exception cref="">無し</exception>
		/// <remarks><para>remarks:</para>
		/// <para>演算中断中にプロパティ、初期値等を変更して動作させ場合等に使用する</para>
		/// </remarks>
		public override bool SetOnlineProperty(McCellModelPropertyIF csCellMdlPropertyInfo)
		{
			// プロパティ設定
			McCellModelPropertyInfo csPrptyInfo = csCellMdlPropertyInfo as McCellModelPropertyInfo;
			if (csPrptyInfo != null)
			{
				// To Do
				// モデルの係数等、必要な情報に対して、引数で与えられたプロパティ情報の内容を読み取って、設定していきます。
				// 但し、実行中に変更して困る情報は設定しないで下さい。
			}
			return true;
		}
		// 
	}
}
